import signal
import setproctitle
import threading


class Station:
    app = None
    room_id_begin = None
    room_id_end = None

    enabled = True

    worker_threads = None

    def __init__(self, app, room_id_begin, room_id_end):
        self.app = app
        self.room_id_begin = room_id_begin
        self.room_id_end = room_id_end
        self.worker_threads = list()

    def run(self):
        setproctitle.setproctitle(self.app.make_proc_name(
            'station:%s-%s' % (self.room_id_begin, self.room_id_end)
        ))
        self._handle_signals()

        self._spawn_worker_threads()

    def _create_worker_thread(self, room_id):
        worker = self.app.worker_class(self, room_id)

        thread = threading.Thread(
            target=worker.run,
        )
        # master不可以独自退出
        thread.daemon = False
        # 标记room_id
        thread.room_id = room_id
        # 用来给task_queue发数据
        thread.worker = worker

        thread.start()

        return thread

    def _spawn_worker_threads(self):
        """
        监控线程
        :return:
        """
        for room_id in range(self.room_id_begin, self.room_id_end+1):
            self.worker_threads.append(self._create_worker_thread(room_id))

        # 线程的重新启动好像会有很多问题，因为worker里面不止一个线程
        # 还是简单直接join，也不考虑拉起了
        # 让各个线程自己去保证enabled==True时不会退出
        for thread in self.worker_threads:
            try:
                thread.join()
            except:
                # 命令行CTRL-C的时候可能抛异常
                pass

    def _handle_signals(self):
        def safe_stop_handler(signum, frame):
            self.enabled = False
            for thread in self.worker_threads:
                if thread and thread.is_alive():
                    # 为了打断get
                    thread.worker.push_task(None)

        # 安全停止
        signal.signal(signal.SIGTERM, safe_stop_handler)
        signal.signal(signal.SIGHUP, signal.SIG_IGN)
        signal.signal(signal.SIGINT, signal.SIG_IGN)

    def __repr__(self):
        return '<%s name: %s, room_id_range: [%r, %r]>' % (
            type(self).__name__, self.app.name, self.room_id_begin, self.room_id_end
        )




import base64
import random

from .easy_aes import EasyAES
from ...share.log import logger


class TokenHelper:

    secret = None

    block_size = 16

    def __init__(self, secret):
        """
        secret: AES key,  must be either 16, 24, or 32 bytes long
        """
        self.secret = self._convert_str_to_bytes(secret)

    def encode(self, src: bytes):
        """
        1. 使用aes_cbc方法将源串src加密成串dst
        2. 之后将iv与dst拼成token。
            token的格式为 前16位 固定为iv，剩下的部分为token。

        3. 之后将token进行base64加密方便http传输

        :param src:
        :return: 加密后base64字符串。为方便传输，类型为str。
        """
        iv = self._convert_str_to_bytes(self._gen_rand_str(self.block_size))

        aes = EasyAES(
            self.secret,
            iv
        )

        encrypt_bytes = aes.encrypt(src)

        packed_bytes = iv + encrypt_bytes

        base64_result = base64.b64encode(packed_bytes)

        return base64_result.decode('utf8')

    def decode(self, token: bytes):
        """
        解析token
        :param token:
        :return: 被加密的bytes。
        """

        packed_bytes = base64.b64decode(token)

        iv = packed_bytes[:self.block_size]
        encrypt_bytes = packed_bytes[self.block_size:]

        aes = EasyAES(
            self.secret,
            iv
        )

        decrypt_bytes = aes.decrypt(encrypt_bytes)

        if not decrypt_bytes:
            logger.info('decrypt fail. token: %s', token)
            return None

        return decrypt_bytes

    def _gen_rand_str(self, length):
        """
        generate secret key，参考django
        不区分大小写，因为mysql的unique不区分大小写
        """
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        return ''.join([random.choice(chars) for i in range(length)])

    def _convert_str_to_bytes(self, src):
        """
        转换str为bytes
        bytes => bytes
        str => bytes
        else => str => bytes
        """

        return src if isinstance(src, bytes) else bytes(src, 'utf8')




from ..share.task import Task
from .client import Client
from ..share import constants


class Trigger(object):

    box_class = None
    client = None

    def __init__(self, box_class, host, port, ensure=True):
        self.box_class = box_class
        self.client = Client((host, port), ensure)

    def write_to_worker(self, room_id, data):
        """
        透传到worker进行处理
        """

        task = Task()
        task.cmd = constants.CMD_WRITE_TO_WORKER
        task.room_id = room_id

        if isinstance(data, self.box_class):
            # 打包
            data = data.pack()
        elif isinstance(data, dict):
            data = self.box_class(data).pack()
            
        task.body = data

        return self._write(task.pack())

    def _write(self, data):
        return self.client.write(data)

    def __repr__(self):
        return '<%s address: %s>' % (
            type(self).__name__, self.client.address
        )
